/*	gtk_alert_messages_display.c - Created by Erik Kuiper
Traindirecor is created by Giampiero Caprino.

This file is part of Train Director

Train Director is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

Train Director is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Train Director; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include "gtk_traindirector_settings.h"
#include "gtk_alert_messages_display.h"
#include "gtk_callbacks.h"

GtkWidget *alert_messages_list = NULL;
GtkWidget *alert_messages_list_text_view = NULL;
GtkWidget *alert_messages_window = NULL;
GtkWidget *alert_messages_window_text_view = NULL;
GtkTextBuffer *alert_messages_text_buffer = NULL;
gboolean alerts_list_visible = FALSE;

void  clear_alert_messages_window(void)
{
  gtk_text_buffer_set_text(alert_messages_text_buffer, "", -1);
}

GtkWidget *create_alerts_view(GtkOrientation orientation,
			      GtkWidget **alert_messages_text_view)
{
  GtkWidget *scrolled_window;
  GtkWidget *box;
  GtkWidget *buttons_bar;

  buttons_bar = gtk_toolbar_new();
  if (orientation == GTK_ORIENTATION_VERTICAL) {
    box = gtk_vbox_new(FALSE,
		       2);
    gtk_toolbar_set_orientation(GTK_TOOLBAR(buttons_bar),
				GTK_ORIENTATION_HORIZONTAL);
  }
  else {
    box = gtk_hbox_new(FALSE,
		       2);
    gtk_toolbar_set_orientation(GTK_TOOLBAR(buttons_bar),
				GTK_ORIENTATION_VERTICAL);
  }


  gtk_toolbar_append_item(GTK_TOOLBAR(buttons_bar),
			  NULL,
			  "Clear alert messages",
			  NULL,
			  gtk_image_new_from_stock(GTK_STOCK_CLEAR,
						   GTK_ICON_SIZE_MENU),
			  G_CALLBACK(clear_alert_messages_window),
			  NULL);
  gtk_box_pack_start(GTK_BOX(box),
		     buttons_bar,
		     FALSE,
		     TRUE,
		     0);

  scrolled_window = gtk_scrolled_window_new(NULL, NULL);
  gtk_box_pack_start(GTK_BOX(box),
		     scrolled_window,
		     TRUE,
		     TRUE,
		     0);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
				 GTK_POLICY_NEVER,
				 GTK_POLICY_ALWAYS);

  *alert_messages_text_view = gtk_text_view_new();
  if (alert_messages_text_buffer)
    gtk_text_view_set_buffer(GTK_TEXT_VIEW(*alert_messages_text_view),
			     alert_messages_text_buffer);
  else
    alert_messages_text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW (*alert_messages_text_view));

  gtk_container_add(GTK_CONTAINER (scrolled_window),
		    *alert_messages_text_view);
  gtk_text_view_set_editable(GTK_TEXT_VIEW(*alert_messages_text_view),
  			     FALSE);
  gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(*alert_messages_text_view),
				   FALSE);

  return box;
}


void    show_alerts_list_cmd(gpointer callback_data,
			     gint callback_action,
			     GtkWidget *parent)
{
  if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(parent))) {
    gtk_widget_show_all(alert_messages_list);
    alerts_list_visible = TRUE;
  }
  else {
    gtk_widget_hide(alert_messages_list);
    alerts_list_visible = FALSE;
    if (!alert_messages_window)
      clear_alert_messages_window();
  }

}

GtkWidget  *create_alert_messages_list(void)
{
  if (!alert_messages_list) 
    alert_messages_list = create_alerts_view(GTK_ORIENTATION_HORIZONTAL,
					     &alert_messages_list_text_view);
  return alert_messages_list;
}

void  destroy_alert_messages_window(GtkWidget *window)
{
  alert_messages_window = NULL;
  if (!alerts_list_visible)
    clear_alert_messages_window();
}

void  create_alert_messages_window(void)
{
  alert_messages_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

  g_signal_connect(alert_messages_window,
		   "destroy",
		   G_CALLBACK(destroy_alert_messages_window),
		   NULL);
  
  gtk_window_set_title((GtkWindow*) alert_messages_window,
		       alert_messages_window_title);
  gtk_window_set_default_size(GTK_WINDOW (alert_messages_window),
			      alert_messages_window_default_width,
			      alert_messages_window_default_height);

  gtk_container_add(GTK_CONTAINER(alert_messages_window), 
		    create_alerts_view(GTK_ORIENTATION_VERTICAL,
				       &alert_messages_window_text_view )); 
  
  gtk_widget_show_all(alert_messages_window);
}

void    show_alerts_window_cmd(void)
{
  if (!alert_messages_window)
    create_alert_messages_window();
}

void    alert_message(char *msg)
{
  GtkTextIter text_iterator;
  if (alert_messages_window ||
      alerts_list_visible) {
    gtk_text_buffer_get_end_iter(alert_messages_text_buffer,
				 &text_iterator);

    if (!g_utf8_validate(msg, -1, NULL)) {
      gchar *new_msg;
      gchar *err_char;
      
      new_msg = g_strdup(msg);
      while (!g_utf8_validate(new_msg, -1, &err_char))
	*err_char = '?';
      
      gtk_text_buffer_insert(alert_messages_text_buffer,
			     &text_iterator,
			     new_msg,
			     -1);
      g_free(new_msg);
    }
    else
      gtk_text_buffer_insert(alert_messages_text_buffer,
			     &text_iterator,
			     msg,
			     -1);
    
    gtk_text_buffer_insert(alert_messages_text_buffer,
			   &text_iterator,
			   "\n",
			   -1);
    
    gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(alert_messages_list_text_view),
				 &text_iterator,
				 0,
				 TRUE,
				 0,
				 1);
  }
}

