/*	gtk_layout_error_display.c - Created by Giampiero Caprino
Modified by Erik Kuiper

This file is part of Train Director

Train Director is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

Train Director is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Train Director; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include "gtk_traindirector_settings.h"

GtkWidget *layout_error_window = NULL;
GtkTextBuffer *layout_error_text_buffer = NULL;
gboolean layout_error_ended = FALSE;

void  destroy_layout_error_window(GtkWidget *window)
{
  layout_error_window = NULL;
}

void  create_layout_error_window(void)
{
  GtkWidget *layout_error_text_view;
  GtkWidget *scrolled_window;

  layout_error_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

  g_signal_connect(layout_error_window,
		   "destroy",
		   G_CALLBACK(destroy_layout_error_window),
		   NULL);
  
  gtk_window_set_title((GtkWindow*) layout_error_window,
		       layout_error_window_title);
  gtk_window_set_default_size(GTK_WINDOW (layout_error_window),
			      layout_error_window_default_width,
			      layout_error_window_default_height);

  scrolled_window = gtk_scrolled_window_new(NULL, NULL);
  gtk_container_add(GTK_CONTAINER(layout_error_window), 
		    scrolled_window); 
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
				 GTK_POLICY_NEVER,
				 GTK_POLICY_ALWAYS);

  layout_error_text_view = gtk_text_view_new();
  layout_error_text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW (layout_error_text_view));
  gtk_container_add(GTK_CONTAINER (scrolled_window),
		    layout_error_text_view);
  gtk_text_view_set_editable(GTK_TEXT_VIEW(layout_error_text_view),
  			     FALSE);
  gtk_widget_show_all(layout_error_window);
}

void    layout_error(char *msg)
{
  GtkTextIter text_iterator;

  if (!layout_error_window)
    create_layout_error_window();
  else if (layout_error_ended) 
    gtk_text_buffer_set_text(layout_error_text_buffer, "", -1);

  layout_error_ended = FALSE;

  gtk_text_buffer_get_end_iter(layout_error_text_buffer,
			       &text_iterator);

  if (!g_utf8_validate(msg, -1, NULL)) {
    gchar *new_msg;
    gchar *err_char;

    new_msg = g_strdup(msg);
    while (!g_utf8_validate(new_msg, -1, &err_char))
      *err_char = '?';

    gtk_text_buffer_insert(layout_error_text_buffer,
			   &text_iterator,
			   new_msg,
			   -1);
    g_free(new_msg);
  }
  else
    gtk_text_buffer_insert(layout_error_text_buffer,
			   &text_iterator,
			   msg,
			   -1);
}

void    end_layout_error(void)
{
  layout_error_ended = TRUE;
}

