/*	gtk_main.c - Created by Giampiero Caprino
Modified by Erik Kuiper

This file is part of Train Director

Train Director is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

Train Director is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Train Director; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include <glib.h>
#include <string.h>
#include <stdio.h>
#include "gtk_traindirector_settings.h"
#include "gtk_main_menubar.h"
#include "gtk_main_field.h"
#include "gtk_main_status_bar.h"
#include "trsim.h"
#include "ask.h"
#include "gtk_alert_messages_display.h"

extern	char	*version;

extern  void  gtk_assign_dialog(Train *t);
extern  void  gtk_select_day_dialog(void);
extern  void  gtk_train_info_dialog(Train *t);
extern  void  gtk_preferences_dialog(void);

int main(int argc, char *argv[])
{
  GtkWidget *main_window;
  GtkWidget *main_window_vbox;
  GtkWidget *field_alerts_paned;
  char	buff[256];

  /* Initialize i18n support */
  gtk_set_locale ();

  /* Initialize the widget set */
  gtk_init (&argc, &argv);

  /*#ifdef unix
	if(getenv("TDHOME"))
	    chdir(getenv("TDHOME"));
  #endif*/

  /* Init simulation */
  trainsim_init();

  /* Create the main window */
  main_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

  /* Set up our GUI elements */
  g_signal_connect(main_window,
		   "destroy",
		   gtk_main_quit,
		   NULL);
  
  //  sprintf(buff, "%s %s", main_window_title, version);
  gtk_window_set_title((GtkWindow*) main_window,
		       main_window_title);
  gtk_window_set_default_size(GTK_WINDOW (main_window),
			      main_window_default_width,
			      main_window_default_height);
  main_window_vbox = gtk_vbox_new(FALSE,
				  1);
  gtk_container_add(GTK_CONTAINER (main_window),
		    main_window_vbox);
  gtk_widget_show(main_window_vbox);

  /* Set all the features in the main window. */
  create_main_menubar(main_window,
		      main_window_vbox);
  create_buttons_bar(main_window_vbox);
  create_status_bar(main_window_vbox);
  create_context_status_bar(main_window_vbox);
  field_alerts_paned = gtk_vpaned_new();
  gtk_paned_add1(GTK_PANED(field_alerts_paned),
		 create_main_field());
  gtk_paned_add2(GTK_PANED(field_alerts_paned),
		 create_alert_messages_list());
  gtk_paned_set_position(GTK_PANED(field_alerts_paned),
			 300);
  gtk_box_pack_start(GTK_BOX(main_window_vbox),
		     field_alerts_paned,
		     TRUE,
		     TRUE,
		     0);
  gtk_widget_show(field_alerts_paned);

  /* Show the application window */
  gtk_widget_show (main_window);

  init_pmaps(); // Why is this not called from trainsim_init?

  assign_dialog = gtk_assign_dialog;
  select_day_dialog = gtk_select_day_dialog;
  train_info_dialog = gtk_train_info_dialog;
  options_dialog = gtk_preferences_dialog;

  /* Enter the main event loop, and wait for user interaction */
  gtk_main ();

  /* The user lost interest */
  return 0;
}

void    main_quit_cmd()
{
  gtk_main_quit();
}

gboolean timer_removed = FALSE;

gint	timer_fired(gpointer data)
{
  if (timer_removed)
    return FALSE;

  click_time();
  return(running);
}

void	make_timer(int msec)
{
  static gint timer_tag;

  if(!msec)
    timer_removed = TRUE;
  else {
    timer_removed = FALSE;
    timer_tag = g_timeout_add(msec,
			      timer_fired,
			      0);
  }
}

void about_window_ok_callback(GtkButton *Button,
			      gpointer  Window)
{
  gtk_widget_destroy(Window);
}

static	void	add_about_line(GtkWidget *box1, char *text)
{
  GtkWidget   *child;
  
  child = gtk_label_new(text);
  gtk_box_pack_start(GTK_BOX(box1), child, TRUE, TRUE, 4);
}

void	about_cmd(void)
{
  GtkWidget *window = NULL;
  GtkWidget *box1,*box2;
  GtkWidget *btnClose;
  
  /* Create modal window (Here you can use any window descendent )*/
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(window), "About Train Director");
  
  /* Set window as modal */
  gtk_window_set_modal(GTK_WINDOW(window),TRUE);
  box1 = gtk_vbox_new(FALSE, 5);
  gtk_container_set_border_width(GTK_CONTAINER(box1), 10);
  gtk_container_add(GTK_CONTAINER(window), box1);
  
  add_about_line(box1, main_window_title);
  
  gtk_box_pack_start(GTK_BOX(box1), gtk_hseparator_new(), FALSE, FALSE, 4);
  add_about_line(box1, "Created by Giampiero Caprino\nSunnyvale, California");
  add_about_line(box1, "Modified by Erik Kuiper");
  add_about_line(box1, "This is a game, and is not intended to\nbe used to actually control train traffic.");
  add_about_line(box1, "This is free software, released under the\nGNU General Public License.\n"
		 "The author declines any responsibility for any damage\nthat might occur from the use of this program.");

  gtk_box_pack_start(GTK_BOX(box1), gtk_hseparator_new(), FALSE, FALSE, 4);
  box2 = gtk_hbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(box1), box2, TRUE, TRUE, 4);
  
  btnClose = gtk_button_new_with_label("Ok");
  gtk_box_pack_start(GTK_BOX(box2), btnClose, TRUE, TRUE, 4);
  g_signal_connect(GTK_OBJECT(btnClose),
		   "clicked",
		   G_CALLBACK(about_window_ok_callback),
		   window);

  g_signal_connect(GTK_OBJECT(window),
		   "destroy",
		   G_CALLBACK(gtk_main_quit),
		   NULL);
  
  gtk_widget_show_all(window);
  
  gtk_main();
}

// To be treated...
void  alert_beep(void)
{
  gdk_beep();
  //  printf("Beep\n");
}

void show_tooltable(void)
{
}

void hide_tooltable(void)
{
}


void hide_itinerary(void)
{
}

void show_itinerary(void)
{
}

int     create_path_window(void)
{
}

int create_tgraph_draw(GtkWidget *window)
{
}

void   update_button(char *cmd, char *lbltxt)
{
}

void    create_train(void)
{
}

void  track_info_dialogue()
{
}

void  new_status_position(void)
{
}

int create_tgraph(void)
{
}

void    select_tool(int i)
{
}

void    track_dialogue(Track *t)
{
}

int screen_width = 1024;
int     ntoolrows = 2;
int     current_tool;
int   status_on_top = 1;
