/*	gtk_main_field.c - Created by Giampiero Caprino
Modified by Erik Kuiper

This file is part of Train Director

Train Director is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

Train Director is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Train Director; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*/

#define PANGO_ENABLE_BACKEND

#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include "gtk_main_field.h"
#include "gtk_traindirector_settings.h"
#include <stdio.h> //For trsim.h
#include "trsim.h"
#include "color.h"
#include <pango/pango.h>
#include <pango/pangox.h>

//#include "itracks.h"	    /* icons for tools */
//#include "iswitch.h"
//#include "isignals.h"
//#include "itools.h"
//#include "iactions.h"

GtkWidget *field;
GdkGC     *field_GC;
GdkPixmap *field_pixmap;
//GtkWidget *field_drawing_area;
void      *x_display;

gboolean  updating_all = FALSE;
extern    int     ignore_cliprect;
gboolean  show_grid = FALSE;

enum rgb_components {RED_COMP, GREEN_COMP, BLUE_COMP};

// Color numbers are defined in the color_* objects.
const int nr_of_colors = 12;
GdkColor color_table[12] = {
  { 0,     0,     0,     0 }, // Black
  { 0, 65535, 65535, 65535 }, // White
  { 0,     0, 65535,     0 }, // Green
  { 0, 65535, 65535,     0 }, // Yellow
  { 0, 65535,     0,     0 }, // Red
  { 0, 65535, 32768,     0 }, // Orange
  { 0, 65535, 32768, 32768 }, // Brown
  { 0, 32768, 32768, 32768 }, // Gray
  { 0, 49152, 49152, 49152 }, // Light gray
  { 0, 16384, 16384, 16384 }, // Dark gray
  { 0,     0,     0, 32768 }, // Blue
  { 0,     0, 65535, 65535 }  // Cyan
};

int field_element_size = default_field_element_size;

int field_element_sizes[] = {8, 9, 10, 12, 15, -1};

PangoContext *normal_label_context;
PangoContext *small_label_context;
PangoFont    *label_font;
GdkColormap  *field_colormap;

void clear_field(void);


/* Create a new backing pixmap of the appropriate size */
static gint field_configure_event(GtkWidget         *widget,
				  GdkEventConfigure *event )
{

  if(field_pixmap)
    return 0;
  //invalidate_field();
  field_pixmap =  gdk_pixmap_new(widget->window,
				 field_width * field_element_size,
				 field_height * field_element_size,
				 -1);

  field_GC = gdk_gc_new(field->window); //maybe not the ideal place.

  x_display = gdk_x11_drawable_get_xdisplay(GDK_DRAWABLE(field_pixmap));
  {
    PangoFontDescription *small_label_font_description;

    normal_label_context = pango_context_new();
    pango_context_set_font_map(normal_label_context,
			       pango_x_font_map_for_display(x_display));

    small_label_context  = pango_context_new();
    pango_context_set_font_map(small_label_context,
			       pango_x_font_map_for_display(x_display));
    small_label_font_description = pango_context_get_font_description(small_label_context);
    pango_font_description_set_size(small_label_font_description,
				    10 * PANGO_SCALE);
    
    pango_context_load_font(small_label_context,
			    small_label_font_description);
    //    printf("Font size: %d\n", pango_font_description_get_size(small_label_font_description)/PANGO_SCALE);
    //    printf("Family: %s\n", pango_font_description_get_family(small_label_font_description));
  }

  label_font = pango_context_load_font(normal_label_context,
				       field->style->font_desc);
  field_colormap = gdk_colormap_new(gdk_visual_get_best(),
				    TRUE);
  gdk_gc_set_colormap(field_GC,
		      field_colormap);

  cliprect.left = 0;
  cliprect.top = 0;
  cliprect.right = field_width;
  cliprect.bottom = field_height;
  clear_field();
  return TRUE;
}

/* Redraw the screen from the backing pixmap */
gint field_expose_event(GtkWidget      *widget,
			GdkEventExpose *event )
{
  gdk_draw_pixmap(widget->window,
		  widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
		  field_pixmap,
		  event->area.x, event->area.y,
		  event->area.x, event->area.y,
		  event->area.width, event->area.height);
  return FALSE;
}

static gint field_motion_notify_event(GtkWidget *widget,
				      GdkEventMotion *event )
{
	int x, y;
	GdkModifierType state;

	if(event->is_hint)
	    gdk_window_get_pointer(event->window, &x, &y, &state);
	else {
	    x = event->x;
	    y = event->y;
	    state = event->state;
	}

	pointer_at(x / field_element_size,
		   y / field_element_size);
	return TRUE;
}

static gint field_button_press_event(GtkWidget      *widget,
				     GdkEventButton *event )
{
	int	x, y;
	GdkModifierType state;

	x = event->x / field_element_size;
	y = event->y / field_element_size;
	
	if(event->button == 1 && field_pixmap) {
	  //	    if(editing)
	  //		track_place(x, y);
	  //	  else
	  track_selected(x, y);
	}
	else if (event->button == 3 && field_pixmap) {
	  track_selected1(x, y);
	}
/*	if(event->button > 1 && field_grid->pixmap) {
	    if(editing_itinerary)
		do_itinerary_dialog(x, y);
	    else if(editing)
		track_properties(x, y);
	    else
		track_selected1(x, y);
		}*/

	return TRUE;
}

GtkWidget *create_main_field(void)
{
  GtkWidget *field_scrolled_window;
  int i;

  field_scrolled_window = gtk_scrolled_window_new(NULL,
						  NULL);
  
  field = gtk_drawing_area_new();
  gtk_widget_set_size_request(field,
			      field_width * field_element_size,
			      field_height * field_element_size);
  gtk_scrolled_window_add_with_viewport((GtkScrolledWindow*) field_scrolled_window,
					field);

  /* Add signals from field to callbacks */
  g_signal_connect(GTK_OBJECT(field),
		   "configure_event",
		   (GtkSignalFunc) field_configure_event,
		   NULL);
  g_signal_connect(GTK_OBJECT(field),
		   "expose_event",
		   (GtkSignalFunc) field_expose_event,
		   NULL);
  
  g_signal_connect(GTK_OBJECT(field),
		   "motion_notify_event",
		   (GtkSignalFunc) field_motion_notify_event,
		   NULL);
  g_signal_connect(GTK_OBJECT(field),
		   "button_press_event",
		   (GtkSignalFunc) field_button_press_event,
		   NULL);

  gtk_widget_set_events(field, 
			GDK_EXPOSURE_MASK
			| GDK_LEAVE_NOTIFY_MASK
			| GDK_BUTTON_PRESS_MASK
			| GDK_POINTER_MOTION_MASK
			| GDK_POINTER_MOTION_HINT_MASK);

  /* Setup colors */
  color_black = 0;
  color_white = 1;
  color_green = 2;
  color_yellow = 3;
  color_red = 4;
  color_orange = 5;
  color_brown = 6;
  color_gray = 7;
  color_lightgray = 8;
  color_darkgray = 9;
  color_blue = 10;
  color_cyan = 11;

  for (i = 0; i < nr_of_colors; ++i){
    gdk_rgb_find_color(gtk_widget_get_colormap(field),
		       &(color_table[i]));
  }

  fieldcolors[COL_BACKGROUND] = color_white;
  fieldcolors[COL_TRACK] = color_black;
  fieldcolors[COL_GRAPHBG] = color_white;
  
  fieldcolors[COL_TRAIN1] = color_orange;
  fieldcolors[COL_TRAIN2] = color_cyan;
  fieldcolors[COL_TRAIN3] = color_blue;
  fieldcolors[COL_TRAIN4] = color_yellow;
  
  gtk_widget_show_all(field_scrolled_window);
  return field_scrolled_window;
};

void	clear_field(void)
{
  //	if(editing)
  //	    invalidate_field();
  //  printf("Clear field. Left: %d Right: %d Top: %d Bottom %d\n",cliprect.left, cliprect.right, cliprect.top, cliprect.bottom);
  gdk_gc_set_foreground(field_GC,
			&color_table[color_white]);
  gdk_draw_rectangle(field_pixmap,
  		     field_GC,
  		     TRUE,
  		     cliprect.left * field_element_size,
		     cliprect.top * field_element_size,
  		     (cliprect.right - cliprect.left + 1) * field_element_size,
		     (cliprect.bottom - cliprect.top + 1) * field_element_size);
}

void	draw_all_pixmap(void)
{
  gdk_draw_pixmap(field->window,
		  field_GC,
		  field_pixmap,
		  cliprect.left * field_element_size,
		  cliprect.top * field_element_size,
		  cliprect.left * field_element_size,
		  cliprect.top * field_element_size,
		  cliprect.right * field_element_size,
		  cliprect.bottom * field_element_size);
}

void	grid_paint(void)
{
  int	x, y;

  gdk_gc_set_foreground(field_GC,
			&color_table[color_black]);
  for(x = cliprect.left; x <= cliprect.right; ++x)
    for(y = cliprect.top; y <= cliprect.bottom; ++y)
      gdk_draw_point(field_pixmap, 
		     field_GC,
		     x * field_element_size,
		     y * field_element_size);
}


void    repaint_all(void)
{
  if(cliprect.top > cliprect.bottom) // !editing && 
    return;			/* no changes since last update */
  if(ignore_cliprect) // || editing)
    clear_field();
  updating_all = TRUE;
  layout_paint(layout);
  if(show_grid)
    grid_paint();
  trains_paint(schedule);
  updating_all = FALSE;
  draw_all_pixmap();
  reset_clip_rect();
}

void	update_rectangle_at(int x, int y)
{
	GdkRectangle update_rect;

	if(updating_all)
	  return;
	x *= field_element_size;
	y *= field_element_size;
	update_rect.x = x;
	update_rect.y = y;
	update_rect.width = field_element_size;
	update_rect.height = field_element_size;
	gtk_widget_draw(field, &update_rect);
}

void    draw_segments(int x0, int y0, SegDir *p, grcolor col)
{
  GdkRectangle update_rect;
  int	x = x0 * field_element_size;
  int	y = y0 * field_element_size;
  int	hx;
  int	hy;

  gdk_gc_set_foreground(field_GC,
			&(color_table[col]));
  hx = field_element_size / 2;
  hy = field_element_size / 2;
  while(*p != SEG_END) {
    switch(*p) {
    case SEG_N:
      gdk_draw_line(field_pixmap, 
		    field_GC,
		    x + hx, y, x + hx, y + hy);
      gdk_draw_line(field_pixmap,
		    field_GC,
		    x + hx + 1, y, x + hx + 1, y + hy);
      break;
    case SEG_NE:
      gdk_draw_line(field_pixmap,
		    field_GC,
		    x + hx, y + hy - 1, x + hx * 2 - 1, y);
      gdk_draw_line(field_pixmap,
		    field_GC,
		    x + hx, y + hy, x + hx * 2, y);
      gdk_draw_line(field_pixmap,
		    field_GC,
		    x + hx, y + hy + 1, x + hx * 2, y + 1);
      break;
    case SEG_E:
      gdk_draw_line(field_pixmap,
		    field_GC,
		    x + hx, y + hy, x + hx * 2, y + hy);
      gdk_draw_line(field_pixmap,
		    field_GC,
		    x + hx, y + hy + 1, x + hx * 2, y + hy + 1);
      break;
    case SEG_SE:
      gdk_draw_line(field_pixmap,
		    field_GC,
		    x + hx + 1, y + hy, x + hx * 2, y + hy * 2 - 1);
      gdk_draw_line(field_pixmap,
		    field_GC,
		    x + hx, y + hy, x + hx * 2, y + hy * 2);
      gdk_draw_line(field_pixmap,
		    field_GC,
		    x + hx, y + hy + 1, x + hx * 2 - 1, y + hy * 2);
      break;
    case SEG_S:
      gdk_draw_line(field_pixmap,
		    field_GC,
		    x + hx, y + hy, x + hx, y + hy * 2);
      gdk_draw_line(field_pixmap,
		    field_GC,
		    x + hx + 1, y + hy, x + hx + 1, y + hy * 2);
      break;
    case SEG_SW:
      gdk_draw_line(field_pixmap,
		    field_GC,
		    x, y + hy * 2 - 1, x + hx - 1, y + hy);
      gdk_draw_line(field_pixmap,
		    field_GC,
		    x, y + hy * 2, x + hx, y + hy);
      gdk_draw_line(field_pixmap,
		    field_GC,
		    x + 1, y + hy * 2, x + hx, y + hy + 1);
      break;
    case SEG_W:
      gdk_draw_line(field_pixmap,
		    field_GC,
		    x, y + hy, x + hx, y + hy);
      gdk_draw_line(field_pixmap,
		    field_GC,
		    x, y + hy + 1, x + hx, y + hy + 1);
      break;
    case SEG_NW:
      gdk_draw_line(field_pixmap,
		    field_GC,
		    x + 1, y, x + hx, y + hy - 1);
      gdk_draw_line(field_pixmap,
		    field_GC,
		    x, y, x + hx, y + hy);
      gdk_draw_line(field_pixmap,
		    field_GC,
		    x, y + 1, x + hx - 1, y + hy);
      break;
    }
    ++p;
  }
  update_rectangle_at(x0, y0);
}

void    draw_element_box(int x0, int y0, grcolor col, grcolor fill_color)
{
  gdk_gc_set_foreground(field_GC,
			&(color_table[col]));
  gdk_draw_rectangle(field_pixmap,
  		     field_GC,
  		     FALSE,
  		     x0 * field_element_size,
  		     y0 * field_element_size,
  		     field_element_size - 1,
  		     field_element_size - 1);

  if (fill_color >= 0) {
    gdk_gc_set_foreground(field_GC,
			  &color_table[fill_color]);
    gdk_draw_rectangle(field_pixmap,
		       field_GC,
		       TRUE,
		       x0 * field_element_size + 1,
		       y0 * field_element_size + 1,
		       field_element_size - 2,
		       field_element_size - 2);
  }
}

void    getcolor_rgb(int col, int *r, int *g, int *b)
{
	if(col < 0 || col >= nr_of_colors)
	    return;
	*r = color_table[col].red   / 256;
	*g = color_table[col].green / 256;
	*b = color_table[col].blue  / 256;
}

void    *get_pixmap(const char **map)
{
  GdkPixmap *pmap;
  GdkPixmap *mask;
  GtkStyle *style;
  
  style = gtk_widget_get_style( field );
  pmap = gdk_pixmap_colormap_create_from_xpm_d(field->window,
					       field_colormap,
					       &mask,
					       NULL, //&style->bg[GTK_STATE_NORMAL],
					       (gchar **)map );
  return((void *)pmap);
}


void    draw_layout(int x0, int y0, VLines *p, grcolor col)
{
  GdkRectangle update_rect;
  int x = x0 * field_element_size;
  int y = y0 * field_element_size;
  
  gdk_gc_set_foreground(field_GC,
			&(color_table[col]));
  
  while(p->x0 >= 0) {
    gdk_draw_line(field_pixmap,
		  field_GC,
		  x + p->x0,
		  y + p->y0,
		  x + p->x1,
		  y + p->y1);
    ++p;
  }
  update_rectangle_at(x0, y0);
}

void    draw_pixmap(int x0, int y0, void *map)
{
  GdkPixmap *pmap = (GdkPixmap *)map;
  GtkStyle *style;
  GdkRectangle update_rect;
  int x = x0 * field_element_size;
  int y = y0 * field_element_size;
  
  //  if(!current_grid->pixmap)
  //    return;
  //  if(!current_grid->gc)
  //    current_grid->gc = gdk_gc_new(current_grid->drawing_area->window);
  style = gtk_widget_get_style(field);
  //  if(current_grid == tools_grid) {
  //    x += HGRID / 2 * tools_grid->hmult;
  //    y += VGRID / 2 * tools_grid->vmult;
  //  }
  if(pmap)
    gdk_draw_drawable(field_pixmap,
		      field_GC,
		      pmap, 
		      0,
		      0,
		      x,
		      y,
		      -1,
		      -1);
  if(updating_all)
    return;
  update_rect.x = x;
  update_rect.y = y;
  update_rect.width = 16;
  update_rect.height = 11;
  gtk_widget_draw(field, &update_rect);
}

void    tr_fillrect(int x, int y)
{
	GtkStyle *style;

	x *= field_element_size;
	y *= field_element_size;
	style = gtk_widget_get_style( field );
	gdk_gc_set_foreground(field_GC,
			      &(color_table[color_white]));
	gdk_draw_rectangle(field_pixmap, 
			   field_GC,
			   TRUE,
			   x,
			   y,
			   field_element_size,
			   field_element_size);
}

extern VLines *n_s_platform_out;
extern VLines *n_s_platform_in;

void	draw_layout_text1(int x, int y, char *text, int small)
{
  GdkRectangle update_rect;
  PangoLayout *pango_layout;
  PangoLayoutLine *pango_line;

  GtkStyle *style;
  PangoFontDescription *font;

  //	y -= is_windows;		    /* fix incompatibility in Win GTK */
  update_rect.x = x * field_element_size;
  update_rect.y = y * field_element_size;
  if(small)
    pango_layout = pango_layout_new(small_label_context);
  else
    pango_layout = pango_layout_new(normal_label_context);
  pango_layout_set_text(pango_layout,
			text,
			-1);
  pango_line = pango_layout_get_line(pango_layout,
				     0);
  gdk_gc_set_foreground(field_GC,
			&(color_table[color_black]));
  pango_x_render_layout_line(x_display,
			     gdk_x11_drawable_get_xid(GDK_DRAWABLE(field_pixmap)),
			     gdk_x11_gc_get_xgc(field_GC),
			     pango_line,
			     x * field_element_size,
			     (y+1) * field_element_size);
	    
  if(updating_all)
    return;

  //  update_rect.width = gdk_string_width(font, text);
  //  update_rect.height = gdk_string_height(font, text);
  gtk_widget_draw(field, &update_rect);
}

void	draw_link(int x0, int y0, int x1, int y1, int color)
{
  int	hx;
  int	hy;
  
  x0 = x0 * field_element_size;
  y0 = y0 * field_element_size;
  x1 = x1 * field_element_size;
  y1 = y1 * field_element_size;
  
  gdk_gc_set_foreground(field_GC,
			&(color_table[color]));
  hx = field_element_size / 2;
  hy = field_element_size / 2;
  gdk_draw_line(field_pixmap,
		field_GC,
		x0 + hx,
		y0 + hy,
		x1 + hx,
		y1 + hy);
}

void    draw_layout_text(int x, int y, char *text)
{
  draw_layout_text1(x, y, text, 0);
}

void    *get_pixmap_file(const char *fname)
{
  GdkPixmap *pmap;
  char	*p;
  char	buff[256];
  GdkColor background_color = color_table[color_white];
  
  strcpy(buff, fname);
  for(p = buff; *p; ++p)
    if(*p >= 'A' && *p <= 'Z')
      *p += ' ';

  pmap = gdk_pixmap_create_from_xpm(field->window,
				    NULL,
				    &background_color,
				    (gchar *)fname );
  if(pmap)
    return((void *)pmap);
  for(p = fname + strlen(fname); p >= fname; --p)
    if(*p == '\\' || *p == '/' || *p == ':') {
      ++p;
      break;
    }
  if(p < fname)
    p = fname;
  pmap = gdk_pixmap_create_from_xpm(field->window,
				    NULL,
				    &background_color,
				    (gchar *)p );
  //  printf("pixmap filename: %s\n", p);
  if(pmap)
    return((void *)pmap);
  //#ifdef unix
  sprintf(buff, "%s/%s", curpath, p);
  pmap = gdk_pixmap_create_from_xpm(field->window,
				    NULL,
				    &background_color,
				    (gchar *)buff);
  //  printf("pixmap filename with path: %s\n", buff);
  //#endif /* unix */
  if(pmap)
    return((void *)pmap);
  if(getenv("TDICONDIR")) {
    sprintf(buff, "%s/%s", getenv("TDICONDIR"), p);
    pmap = gdk_pixmap_create_from_xpm(field->window,
				      NULL,
				      &background_color,
				      (gchar *)buff );
    //    printf("pixmap filename: %s\n", buff);
    if(pmap)
      return((void *)pmap);
  }
  //#ifdef unix
  /*tdicons in scenario directory */
  sprintf(buff, "%s/tdicons/%s", curpath, p);
  pmap = gdk_pixmap_create_from_xpm(field->window,
				    NULL,
				    &background_color,
				    (gchar *)buff );
  //printf("pixmap filename: %s\n", buff);
  if(pmap)
    return((void *)pmap);
  if(getenv("TDHOME") != NULL) { /* in TDHOME/tdicons */
    sprintf(buff,"%s/tdicons/%s", getenv("TDHOME"), p);
    
    pmap = gdk_pixmap_create_from_xpm(field->window,
				      NULL,
				      &background_color,
				      (gchar *)buff );
    //    printf("pixmap filename: %s\n", buff);
  } else {
    sprintf(buff, "tdicons/%s", p);
    pmap = gdk_pixmap_create_from_xpm(field->window,
				      NULL,
				      &background_color,
				      (gchar *)buff );
    //    printf("pixmap filename: %s\n", buff);
  }
  /*#else
	sprintf(buff, "tdicons/%s", p);
	pmap = gdk_pixmap_create_from_xpm(current_grid->drawing_area->window,
		    &mask, &style->bg[GTK_STATE_NORMAL], (gchar *)buff );
	if(pmap)
	    return((void *)pmap);
	sprintf(buff, "C:/tdicons/%s", p);
	pmap = gdk_pixmap_create_from_xpm(current_grid->drawing_area->window,
		    &mask, &style->bg[GTK_STATE_NORMAL], (gchar *)buff );
		    #endif*/
  {
    gchar buff[1024];

    sprintf(buff, "Could not open: %s\n", fname);
    layout_error(buff);
  }
  return((void *)pmap);
}

void    zoom_in_cmd(void)
{
  int i;

  for (i = 0; !(field_element_sizes[i] == -1 || 
		field_element_sizes[i] > field_element_size) ; ++i);
  if (field_element_sizes[i] == -1)
    return;
  field_element_size = field_element_sizes[i];

  // reset and redraw field
  // Add resize of field if needed.
  invalidate_field();
  repaint_all();
}

void    zoom_out_cmd(void)
{
  int i;

  for (i = 0; !(field_element_sizes[i] == -1 || 
		field_element_sizes[i] >= field_element_size) ; ++i);
  if (i == 0)
    return;
  field_element_size = field_element_sizes[i-1];

  // reset and redraw field
  // Add resize of field if needed.
  invalidate_field();
  repaint_all();
}

void    zoom_100_cmd(void)
{
  int i;

  if (default_field_element_size == field_element_size)
    return;
  field_element_size = default_field_element_size;

  // reset and redraw field
  // Add resize of field if needed.
  invalidate_field();
  repaint_all();
}


