/*	gtk_main_files.c - Created by Giampiero Caprino
Modified by Erik Kuiper

This file is part of Train Director

Train Director is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

Train Director is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Train Director; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include <string.h>
#include <stdio.h>
#include "trsim.h"

/* The file selection widget and the string to store the chosen filename */

typedef struct {
  GtkWidget *file_selector;
  gchar *selected_filename;
} FileDialogData;  

void store_filename (GtkFileSelection *selector, 
		     gpointer dialog_data) {
  strcpy(((FileDialogData*)dialog_data)->selected_filename,
	 gtk_file_selection_get_filename (GTK_FILE_SELECTION (((FileDialogData*) dialog_data)->file_selector)));
  curpath = g_dirname(((FileDialogData*)dialog_data)->selected_filename);
}

void file_dialog (const char* title,
		  char *dst) {
  FileDialogData open_dialog_data;
  char temp_path[1024];

  open_dialog_data.selected_filename = dst;
  /* Create the selector */
  open_dialog_data.file_selector = gtk_file_selection_new (title);
  if (curpath)
    sprintf(temp_path, "%s/%s", curpath, dst);
  else
    sprintf(temp_path, "%s", dst);
  dst[0] = 0;

  gtk_file_selection_set_filename(GTK_FILE_SELECTION(open_dialog_data.file_selector),
				  temp_path);

  gtk_window_set_modal(GTK_WINDOW(open_dialog_data.file_selector),
		       TRUE);
  
  g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (open_dialog_data.file_selector)->ok_button),
		    "clicked",
		    G_CALLBACK (store_filename),
		    (gpointer) &open_dialog_data);
  
  /* Ensure that the dialog box is destroyed when the user clicks a button. */
  
  g_signal_connect_after (GTK_OBJECT (GTK_FILE_SELECTION (open_dialog_data.file_selector)->ok_button),
			  "clicked",
			  G_CALLBACK (gtk_main_quit), 
			  NULL); 
  
  g_signal_connect_after (GTK_OBJECT (GTK_FILE_SELECTION (open_dialog_data.file_selector)->cancel_button),
			  "clicked",
			  G_CALLBACK (gtk_main_quit),
			  NULL); 
  
  gtk_widget_show (open_dialog_data.file_selector);
  
  gtk_main();

  gtk_widget_destroy(open_dialog_data.file_selector);
}

void    open_cmd(GtkWidget *parent,
		 gpointer  data)
{
  char	*p;
  char	buff[256];
  char  file_name[256]; /* Slightly unsafe if people start to use very long paths or names */
  
  file_name[0] = '\0';
  file_dialog("Open File",
	      file_name);
  if(file_name[0]) {
    if(strstr(file_name, ".sav")){
      /* Included is logic for temporarily setting curpath to the path of the 
       * track that is loaded. This is done to be able to load icons that
       * stored in the track's directory. 
       * This logic should be moved to the restore comand.
       */
      FILE *save_file;
      char restore_directory[256];
      char *old_path;
      int  i;
      
      if (save_file = fopen(file_name, "r")) {
	fgets(restore_directory,
	      256,
	      save_file);
	fclose(save_file);
	for(i = strlen(restore_directory) - 1;
	    restore_directory[i] != '/' &&
	      restore_directory[i] != '\\';
	    --i);
	restore_directory[i] = '\0';

	old_path = curpath;
	curpath = restore_directory;

	sprintf(buff, "restore %s", file_name);
	remove_ext(buff);
	trainsim_cmd(buff);

	curpath = old_path;
      }
      else
	error("Selected saved file could not be opened");
    }
    else {
      sprintf(buff, "open %s", file_name);
      remove_ext(buff);
      trainsim_cmd(buff);
    }
  }
}

void  save_cmd(GtkWidget *parent,
	       gpointer  data)
{
  char buff[256];
  char file_name[256];  /* Slightly unsafe if people start to use very long paths or names */
  char *ext_start;

  if (schedule) { // Check that a game is ongoing.
    file_name[0] = '\0';
    file_dialog("Save File",
		file_name);
    if (file_name[0]){
      if(ext_start = strstr(file_name, ".sav"))
	*ext_start = 0;
      sprintf(buff, "savegame %s", file_name);
      trainsim_cmd(buff);
    }
  }
  else
    error("No game is ongoing");
}

int   openFileDialog(char *dst)
{
  file_dialog("Select File",
	      dst);
  if (dst[0])
    return 1;
  else
    return 0;
}

int	openImageDialog(char *dst)
{
  return openFileDialog(dst);
}
