/*	gtk_main_menubar.c - Created by Giampiero Caprino
Modified by Erik Kuiper

This file is part of Train Director

Train Director is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

Train Director is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Train Director; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include "gtk_callbacks.h"
#include <stdio.h> //For trsim.h
#include "trsim.h"

static GtkItemFactoryEntry menu_items[] = {
  { "/_File",                   NULL,      NULL, 0, "<Branch>" },
  { "/File/_Open",      "<control>O",  open_cmd, 0, NULL },
  { "/File/_Save Game", "<control>S",  save_cmd, 0, NULL },
  /*{ "/File/_Restore", NULL,	    restore_cmd, 0, NULL },
#if 0
  { "/File/Save _As", NULL,         NULL, 0, NULL },
#endif
  { "/File/sep0",     NULL,         NULL, 0, "<Separator>" },
  { "/File/_Log",     NULL,	    log_cmd, 0, NULL },
  { "/File/_Replay",  NULL,	    replay_cmd, 0, NULL },
  { "/File/sep1",     NULL,         NULL, 0, "<Separator>" },*/
  { "/File/About...",          NULL,     about_cmd, 0, NULL },
  { "/File/sep1",              NULL,          NULL, 0, "<Separator>" },
  { "/File/_Quit",     "<control>Q", main_quit_cmd, 0, NULL },
  { "/_Edit",                  NULL,          NULL, 0, "<Branch>" },
  /*{ "/Edit/_Edit",    "<control>E",	    edit_cmd, 0, NULL },
  { "/Edit/_New Train", NULL,	    new_train_cmd, 0, NULL },
  { "/Edit/Itinerary", NULL,	    itin_cmd, 0, NULL },
  { "/Edit/_Save Layout", NULL,	    save_cmd, 0, NULL },*/
  { "/Edit/_Preferences",NULL,	    preferences_cmd, 0, NULL },
  /*{ "/Edit/_New",     "<control>N", new_cmd, 0, NULL },
  { "/Edit/_Info",    "<control>I", info_cmd, 0, NULL }, */
  { "/_View",                   NULL,                   NULL, 0, "<Branch>" },
  { "/View/Zoom 100%",          NULL,           zoom_100_cmd, 0, NULL },
  { "/View/Zoom in",            NULL,            zoom_in_cmd, 0, NULL },
  { "/View/Zoom out",           NULL,           zoom_out_cmd, 0, NULL },
  { "/View/sep1",               NULL,                   NULL, 0, "<Separator>" },
  { "/View/Buttons Bar",        NULL,   show_buttons_bar_cmd, 0, "<CheckItem>" },
  { "/View/Schedule",           NULL,      show_schedule_cmd, 0, NULL },
  { "/View/Alerts Window",      NULL, show_alerts_window_cmd, 0, NULL },
  { "/View/Alerts List",        NULL,   show_alerts_list_cmd, 0, "<CheckItem>" },
  { "/_Run",                    NULL,                   NULL, 0, "<Branch>" },
  { "/Run/Start Stop",          NULL,              start_cmd, 0, NULL },
  { "/Run/_Fast",       "<control>X",               fast_cmd, 0, NULL },
  { "/Run/_Slow",       "<control>Y",               slow_cmd, 0, NULL },
  /*  { "/Run/View _Blocks", "<control>B",	blocks_cmd, 0, NULL },
  { "/Run/View S_peed", "<control>P",	speed_cmd, 0, NULL },
  { "/Run/View T_raditional", "<control>T", traditional_cmd, 0, NULL },
  { "/Run/View _Graph", "<control>G",	graph_cmd, 0, NULL },
  { "/Run/Restart", NULL,	restart_cmd, 0, NULL },
  { "/Run/S_tation schedule", NULL,	stationsched_cmd, 0, NULL },
  { "/Run/Set sig. to green", NULL,	open_sigs_cmd, 0, NULL },
  { "/Run/Performance", NULL,	performance_cmd, 0, NULL },
  { "/Run/_Alert Sound", "<control>A",	alert_cmd, 0, NULL },
#if 0
  { "/_Help",         NULL,         NULL, 0, "<LastBranch>" },
  { "/_Help/About",   NULL,         NULL, 0, NULL },
#endif
  { "/_Erik",         NULL,         NULL, 0, "<Branch>" },
  { "/Erik/Incoming Trains",   NULL,         incoming_trains_cmd, 0, NULL },*/
};

GtkWidget *start_stop_image_container;
GtkWidget *start_image;
GtkWidget *stop_image;

void create_main_menubar(GtkWidget *main_window,
		      GtkWidget *parent)
{  
  GtkItemFactory *menubar_factory;
  GtkAccelGroup *menubar_accel_group;
  GtkWidget *menubar;
  GtkWidget *Check_Item;
  gint nr_menu_items = sizeof (menu_items) / sizeof (menu_items[0]);

  menubar_accel_group = gtk_accel_group_new ();

  /* This function initializes the item factory.
     Param 1: The type of menu - can be GTK_TYPE_MENU_BAR, GTK_TYPE_MENU,
              or GTK_TYPE_OPTION_MENU.
     Param 2: The path of the menu.
     Param 3: A pointer to ga gtk_accel_group.  The item factory sets up
              the accelerator table while generating menus.
  */

  menubar_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR,
					  "<main>", 
					  menubar_accel_group);

  /* This function generates the menu items. Pass the item factory,
     the number of items in the array, the array itself, and any
     callback data for the the menu items. */
  gtk_item_factory_create_items (menubar_factory,
				 nr_menu_items,
				 menu_items,
				 NULL);

  /* Attach the new accelerator group to the window. */
  gtk_window_add_accel_group ((GtkWindow*) main_window,
			      menubar_accel_group);

  /* Select show buttons bar */
  Check_Item = gtk_item_factory_get_item(menubar_factory,
					 "/View/Buttons Bar");
  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(Check_Item),
				 TRUE);

  menubar = gtk_item_factory_get_widget (menubar_factory, "<main>");
  gtk_box_pack_start(GTK_BOX(parent),
		     menubar,
		     FALSE,
		     TRUE,
		     0);
  gtk_widget_show(menubar);

}

GtkWidget *buttons_bar = NULL;

void  create_buttons_bar(GtkWidget *parent)
{
  GtkWidget *start_stop_button;
  GtkWidget *toggle_button;

  buttons_bar = gtk_toolbar_new();
  gtk_toolbar_set_style(GTK_TOOLBAR(buttons_bar),
			GTK_TOOLBAR_ICONS);

  start_image = g_object_ref(gtk_image_new_from_stock(GTK_STOCK_APPLY,
						      GTK_ICON_SIZE_MENU));
  stop_image =  g_object_ref(gtk_image_new_from_stock(GTK_STOCK_STOP,
						      GTK_ICON_SIZE_MENU));
  gtk_widget_show(stop_image);

  start_stop_button = gtk_toolbar_append_item(GTK_TOOLBAR(buttons_bar),
					      NULL,
					      "Starts/stops the simulation",
					      NULL,
					      start_image,
					      start_cmd,
					      NULL);
  gtk_toolbar_append_item(GTK_TOOLBAR(buttons_bar),
			  NULL,
			  "Increases simulation speed",
			  NULL,
			  gtk_image_new_from_stock(GTK_STOCK_ADD,
						   GTK_ICON_SIZE_MENU),
			  fast_cmd,
			  NULL);
  gtk_toolbar_append_item(GTK_TOOLBAR(buttons_bar),
			  NULL,
			  "Reduces simulation speed",
			  NULL,
			  gtk_image_new_from_stock(GTK_STOCK_REMOVE,
						   GTK_ICON_SIZE_MENU),
			  slow_cmd,
			  NULL);
  gtk_toolbar_append_item(GTK_TOOLBAR(buttons_bar),
			  NULL,
			  "Show train schedule dialog",
			  NULL,
			  gtk_image_new_from_stock(GTK_STOCK_JUSTIFY_FILL,
						   GTK_ICON_SIZE_MENU),
			  show_schedule_cmd,
			  NULL);

  gtk_toolbar_append_space(GTK_TOOLBAR(buttons_bar));

  gtk_toolbar_append_item(GTK_TOOLBAR(buttons_bar),
			  NULL,
			  "Zoom in",
			  NULL,
			  gtk_image_new_from_stock(GTK_STOCK_ZOOM_IN,
						   GTK_ICON_SIZE_MENU),
			  G_CALLBACK(zoom_in_cmd),
			  NULL);
  gtk_toolbar_append_item(GTK_TOOLBAR(buttons_bar),
			  NULL,
			  "Zoom out",
			  NULL,
			  gtk_image_new_from_stock(GTK_STOCK_ZOOM_OUT,
						   GTK_ICON_SIZE_MENU),
			  G_CALLBACK(zoom_out_cmd),
			  NULL);
  gtk_toolbar_append_item(GTK_TOOLBAR(buttons_bar),
			  NULL,
			  "Set standard size",
			  NULL,
			  gtk_image_new_from_stock(GTK_STOCK_ZOOM_100,
						   GTK_ICON_SIZE_MENU),
			  G_CALLBACK(zoom_100_cmd),
			  NULL);

  /*  gtk_toolbar_append_space(GTK_TOOLBAR(buttons_bar));

  toggle_button = gtk_toggle_button_new();
  gtk_button_set_relief(GTK_BUTTON(toggle_button),
			GTK_RELIEF_NONE);
  gtk_container_add(GTK_CONTAINER(toggle_button),
		    gtk_image_new_from_stock(GTK_STOCK_DIALOG_WARNING,
					     GTK_ICON_SIZE_MENU));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle_button),
  			       alarms_in_main_window);
  g_signal_connect(toggle_button,
  		   "toggled",
  		   G_CALLBACK(alert_button_toggle),
  		   NULL);
  gtk_toolbar_append_element(GTK_TOOLBAR(buttons_bar),
			     GTK_TOOLBAR_CHILD_WIDGET, //GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
			     toggle_button,
			     NULL,
			     "Displays/hides the alerts list in the main window",
			     NULL,
			     NULL,
			     NULL,
			     NULL);*/

  gtk_box_pack_start(GTK_BOX(parent),
		     buttons_bar,
		     FALSE,
		     TRUE,
		     0);
  gtk_widget_show_all(buttons_bar);

  {
    GList *temp_list;

    temp_list = gtk_container_get_children(GTK_CONTAINER(start_stop_button));
    start_stop_image_container = g_list_nth_data(temp_list,
						 0);
    g_list_free(temp_list);
  }
}

void set_start_stop_icon (void)
{
  if(running){
    //    if( != start_image) { // Should add sanity check.
      gtk_container_remove(GTK_CONTAINER(start_stop_image_container),
			   start_image);
      gtk_container_add(GTK_CONTAINER(start_stop_image_container),
			stop_image);
      //    }
  }
  else {
    //    if( != stop_image) {
      gtk_container_remove(GTK_CONTAINER(start_stop_image_container),
			   stop_image);
      gtk_container_add(GTK_CONTAINER(start_stop_image_container),
			start_image);
      //    }
  }
}

void    show_buttons_bar_cmd(gpointer callback_data,
			     gint callback_action,
			     GtkWidget *parent)
{
  if (buttons_bar)
    if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(parent))) {
      gtk_widget_show_all(buttons_bar);
    }
    else {
      gtk_widget_hide(buttons_bar);
    }

}
