/*	gtk_preferences_dialog.h - Created by Giampiero Caprino
Modified by Erik Kuiper

This file is part of Train Director

Train Director is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

Train Director is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Train Director; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*/

#include <stdio.h>  /* for trsim.h */
#include "gtk_callbacks.h"
#include "trsim.h"
#include "gtk_alert_messages_display.h"

void  preferences_cmd(void)
{
  trainsim_cmd("options");
}

static	struct	_opt {
  GtkWidget   *sig;
  GtkWidget   *blocks;
  GtkWidget   *speeds;
  GtkWidget   *alert;
  GtkWidget   *seconds;
  GtkWidget   *statustop;
  GtkWidget   *terse;
  GtkWidget   *grid;
  GtkWidget   *siglink;
  GtkWidget   *window;
  GtkWidget   *hard;
  //  GtkWidget   *alert_window;
} opts;

static	void	opts_continue(gpointer button, gpointer data)
{
  if(!opts.window)
    return;
  signal_traditional = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(opts.sig)) == FALSE;
  show_blocks = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(opts.blocks)) == TRUE;
  beep_on_alert = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(opts.alert)) == TRUE;
  show_speeds = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(opts.speeds)) == TRUE;
  show_seconds = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(opts.seconds)) == TRUE;
  /*  status_on_top = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(opts.statustop)) == TRUE;*/
  terse_status = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(opts.terse)) == FALSE;
  show_grid = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(opts.grid)) == TRUE;
  auto_link = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(opts.siglink)) == TRUE;
  hard_counters = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(opts.hard)) == TRUE;
  //  alerts_window_active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(opts.alert_window)) == TRUE;
  gtk_widget_destroy(opts.window);
  opts.window = 0;
  gtk_main_quit();
}

void  gtk_preferences_dialog(void)
{
  GtkWidget *preferences_window = NULL;
  GtkWidget *box1,*box2;
  GtkWidget *button;
  GtkWidget *child;
  char	buff[256];

  /* Create modal window (Here you can use any window descendent )*/
  preferences_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(preferences_window), "Preferences");

  /* Set window as modal */
  gtk_window_set_modal(GTK_WINDOW(preferences_window), TRUE);

  /* Create widgets */
  box1 = gtk_vbox_new(FALSE, 5);
  gtk_container_add(GTK_CONTAINER(preferences_window), box1);
  gtk_container_set_border_width (GTK_CONTAINER (box1), 10);

  opts.terse = gtk_check_button_new_with_label("Full status info");
  gtk_box_pack_start(GTK_BOX(box1), opts.terse, TRUE, TRUE, 4);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(opts.terse), !terse_status);

  /*  opts.statustop = gtk_check_button_new_with_label("Status line at top of window");
  gtk_box_pack_start(GTK_BOX(box1), opts.statustop, TRUE, TRUE, 4);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(opts.statustop), status_on_top);*/

  opts.alert = gtk_check_button_new_with_label("Alert sound on");
  gtk_box_pack_start(GTK_BOX(box1), opts.alert, TRUE, TRUE, 4);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(opts.alert), beep_on_alert);

  opts.speeds = gtk_check_button_new_with_label("View speed limits");
  gtk_box_pack_start(GTK_BOX(box1), opts.speeds, TRUE, TRUE, 4);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(opts.speeds), show_speeds);

  opts.siglink = gtk_check_button_new_with_label("Automatically link signals");
  gtk_box_pack_start(GTK_BOX(box1), opts.siglink, TRUE, TRUE, 4);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(opts.siglink), auto_link);

  opts.grid = gtk_check_button_new_with_label("Show grid");
  gtk_box_pack_start(GTK_BOX(box1), opts.grid, TRUE, TRUE, 4);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(opts.grid), show_grid);

  opts.blocks = gtk_check_button_new_with_label("View long blocks");
  gtk_box_pack_start(GTK_BOX(box1), opts.blocks, TRUE, TRUE, 4);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(opts.blocks), show_blocks);

  opts.seconds = gtk_check_button_new_with_label("Show seconds on clock");
  gtk_box_pack_start(GTK_BOX(box1), opts.seconds, TRUE, TRUE, 4);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(opts.seconds), show_seconds);

  opts.sig = gtk_check_button_new_with_label("Italian signals");
  gtk_box_pack_start(GTK_BOX(box1), opts.sig, TRUE, TRUE, 4);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(opts.sig), !signal_traditional);

  opts.hard = gtk_check_button_new_with_label("Hard performance tracking");
  gtk_box_pack_start(GTK_BOX(box1), opts.hard, TRUE, TRUE, 4);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(opts.hard), hard_counters);

  /*  opts.alert_window = gtk_check_button_new_with_label("Show alerts in separate window");
  gtk_box_pack_start(GTK_BOX(box1), opts.alert_window, TRUE, TRUE, 4);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(opts.alert_window), alerts_window_active);
  */
  gtk_box_pack_start(GTK_BOX(box1), gtk_hseparator_new(), TRUE, TRUE, 0);
  button = gtk_button_new_with_label ("Ok");
  g_signal_connect_object (button,
			   "clicked",
			   G_CALLBACK(opts_continue),
			   preferences_window,
			   0);
  gtk_box_pack_start (GTK_BOX (box1), button, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (button);
  opts.window = preferences_window;
  gtk_widget_show_all(preferences_window);
  gtk_main();
}
