/*	gtk_schedule.c - Created by Giampiero Caprino
Modified by Erik Kuiper

This file is part of Train Director

Train Director is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

Train Director is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Train Director; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include <stdio.h>
#include "gtk_traindirector_settings.h"
#include "trsim.h"

GtkWidget *schedule_window;

typedef enum {FULL_SCHEDULE,
	      WAITING_SCHEDULE,
	      ACTIVE_SCHEDULE,
	      ASSIGN_SCHEDULE,
	      NR_SCHEDULES} TrainListViews;
#define NR_TRAIN_LIST_VIEWS 3

GtkWidget *schedule_list[NR_SCHEDULES];
GtkWidget *selected_train_list;
GtkWidget *train_info_element[4];

GtkListStore *schedule_data[NR_SCHEDULES];
GtkListStore *train_schedule_data;
GtkCellRenderer *schedule_text_renderer;

typedef enum {ENTRY_TIME,
	      ENTRY_POINT,
	      EXIT_POINT,
	      EXIT_TIME,
	      TRAIN_ID,
	      CURRENT_SPEED,
	      MINUTES_DELAYED,
	      MINUTES_LATE,
	      TRAIN_STATUS,
	      NR_SCHEDULE_TEXT_COLUMNS} ScheduleTextColumns;

static gchar *schedule_colum_titles[] = {"Arrival",
					 "From",
					 "To",
					 "Arrival",
					 "Train",
					 "Speed",
					 "Min.Del",
					 "Min.Late",
					 "Status"};

GtkTreeIter schedule_row_iter[NR_SCHEDULES][1000]; //Change to dynamic allocation.

GtkTreePath *assign_selected = NULL;
Train *selected_train = NULL;

void    gr_update_schedule(Train *t, int i)
{
  GtkTreePath *row_to_collapse;
  gboolean collapsed;
  gchar *path_string;

  if (!schedule_window)
    return;

  //  if(schedule_data[FULL_SCHEDULE])
  // Update FULL_SCHEDULE
  gtk_list_store_set(schedule_data[FULL_SCHEDULE],
		     &(schedule_row_iter[FULL_SCHEDULE][i]),
		     CURRENT_SPEED,   current_speed,
		     MINUTES_DELAYED, current_delay,
		     MINUTES_LATE,    current_late,
		     TRAIN_STATUS,    current_status,
		     -1);

  //  if(schedule_data[WAITING_SCHEDULE] && 
  // Update WAITING_SCHEDULE
  if(schedule_row_iter[WAITING_SCHEDULE][i].stamp &&
     t->status != train_READY) { //Remove train as waiting
    gtk_list_store_remove(schedule_data[WAITING_SCHEDULE],
			  &(schedule_row_iter[WAITING_SCHEDULE][i]));
    schedule_row_iter[WAITING_SCHEDULE][i].stamp = 0;
  }

  //  if(schedule_data[ACTIVE_SCHEDULE]) {
  // Update ACTIVE_SCHEDULE
  if(schedule_row_iter[ACTIVE_SCHEDULE][i].stamp) {
    if(t->status == train_READY ||
       t->status == train_DERAILED ||
       t->status == train_ARRIVED) { // remove train.
      gtk_list_store_remove(schedule_data[ACTIVE_SCHEDULE],
			    &(schedule_row_iter[ACTIVE_SCHEDULE][i]));
      schedule_row_iter[ACTIVE_SCHEDULE][i].stamp = 0;
    }
    else // update data
      gtk_list_store_set(schedule_data[ACTIVE_SCHEDULE],
			 &(schedule_row_iter[ACTIVE_SCHEDULE][i]),
			 CURRENT_SPEED,   current_speed,
			 MINUTES_DELAYED, current_delay,
			 MINUTES_LATE,    current_late,
			 TRAIN_STATUS,    current_status,
			 -1);
  }
  else if (!(t->status == train_READY ||
	     t->status == train_DERAILED ||
	     t->status == train_ARRIVED)) { //Add train
    print_train_info(t);
    gtk_list_store_append(schedule_data[ACTIVE_SCHEDULE],            // Should be sorted
			  &(schedule_row_iter[ACTIVE_SCHEDULE][i]));
    gtk_list_store_set(schedule_data[ACTIVE_SCHEDULE],
		       &(schedule_row_iter[ACTIVE_SCHEDULE][i]),
		       ENTRY_TIME,      disp_columns[ENTRY_TIME],
		       ENTRY_POINT,     disp_columns[ENTRY_POINT],
		       EXIT_POINT,      disp_columns[EXIT_POINT],
		       EXIT_TIME,       disp_columns[EXIT_TIME],
		       TRAIN_ID,        disp_columns[TRAIN_ID],
		       CURRENT_SPEED,   disp_columns[CURRENT_SPEED],
		       MINUTES_DELAYED, disp_columns[MINUTES_DELAYED],
		       MINUTES_LATE,    disp_columns[MINUTES_LATE],
		       TRAIN_STATUS,    disp_columns[TRAIN_STATUS],
		       -1);
  }
}

void create_schedule_list(TrainListViews view,
			  Train *old_train)
{
  GtkTreeIter  line_pointer;
  Train* t;
  gint i;

  if (schedule_data[view] &&
      GTK_IS_LIST_STORE(schedule_data[view]))
    gtk_list_store_clear(schedule_data[view]);
  else
    schedule_data[view] = gtk_list_store_new(9,
					     G_TYPE_STRING,
					     G_TYPE_STRING,
					     G_TYPE_STRING,
					     G_TYPE_STRING,
					     G_TYPE_STRING,
					     G_TYPE_STRING,
					     G_TYPE_STRING,
					     G_TYPE_STRING,
					     G_TYPE_STRING);
  i = 0;
  for (t = schedule; t; t = t->next, ++i) {
    if (view == ASSIGN_SCHEDULE &&
	(!(t->status == train_READY &&   // Skip trains that do not start from this station.
	   sameStation(old_train->position->station, t->entrance)) ||
	 (t->status == train_READY &&    // Skip trains that do not run today
	  t->days && 
	  run_day && 
	  !(t->days & run_day))))
      continue;
    if(!t->entrance){ // To match numbering that is provided when gr_update_schedule is called.
      --i;
      continue;
    }

    if (view == ACTIVE_SCHEDULE &&
	(t->status == train_READY ||         // Skip trains that are not running
	 t->status == train_DERAILED ||
	 t->status == train_ARRIVED)) {
      schedule_row_iter[view][i].stamp =  0;
      continue;
    }
    else if (view == WAITING_SCHEDULE &&
	     (t->status != train_READY ||     // Skip trains that are not waiting
	      (t->status == train_READY &&    // Skip trains that do not run today
	       t->days && 
	       run_day && 
	       !(t->days & run_day)))){
      schedule_row_iter[view][i].stamp =  0;
      continue;
    }

    print_train_info(t);
    gtk_list_store_append(schedule_data[view],
			  &(schedule_row_iter[view][i]));
    gtk_list_store_set(schedule_data[view],
		       &(schedule_row_iter[view][i]),
		       ENTRY_TIME,      disp_columns[ENTRY_TIME],
		       ENTRY_POINT,     disp_columns[ENTRY_POINT],
		       EXIT_POINT,      disp_columns[EXIT_POINT],
		       EXIT_TIME,       disp_columns[EXIT_TIME],
		       TRAIN_ID,        disp_columns[TRAIN_ID],
		       CURRENT_SPEED,   disp_columns[CURRENT_SPEED],
		       MINUTES_DELAYED, disp_columns[MINUTES_DELAYED],
		       MINUTES_LATE,    disp_columns[MINUTES_LATE],
		       TRAIN_STATUS,    disp_columns[TRAIN_STATUS],
		       -1);
    }
}

void	fill_schedule(Train *t, int assign)
{
  gint i;

  if (!assign && t == schedule)
    for (i = 0; i < NR_TRAIN_LIST_VIEWS; ++i)
      create_schedule_list(i,
			   NULL);
  else
    printf("Fill schedule unexpected input.\n");
}

void  full_button_toggle(GtkToggleButton *button)
{
  if(gtk_toggle_button_get_active(button))
    gtk_widget_show_all(schedule_list[FULL_SCHEDULE]);
  else
    gtk_widget_hide(schedule_list[FULL_SCHEDULE]);

}

void  active_button_toggle(GtkToggleButton *button)
{
  if(gtk_toggle_button_get_active(button))
    gtk_widget_show_all(schedule_list[ACTIVE_SCHEDULE]);
  else
    gtk_widget_hide(schedule_list[ACTIVE_SCHEDULE]);

}

void  waiting_button_toggle(GtkToggleButton *button)
{
  if(gtk_toggle_button_get_active(button))
    gtk_widget_show_all(schedule_list[WAITING_SCHEDULE]);
  else
    gtk_widget_hide(schedule_list[WAITING_SCHEDULE]);

}

void  selected_train_button_toggle(GtkToggleButton *button)
{
  if(gtk_toggle_button_get_active(button))
    gtk_widget_show_all(selected_train_list);
  else
    gtk_widget_hide(selected_train_list);

}

void save_schedule_cmd()
{
  save_schedule_status();
}

void save_train_cmd()
{
  if (selected_train)
    train_print(selected_train);
  else
    error("No train is selected.");
}

extern       void    *e_train_pmap[4];

void  create_schedule_button_bar(GtkWidget *parent)
{
  GtkWidget *buttons_bar;
  GtkWidget *toggle_button;

  GdkPixbuf   *save_icon;
  GdkPixmap   *train_save_icon;
  GdkGC       *icon_gc;
  GtkWidget   *train_save_image;
  GdkColormap *system_colormap;
  int         icon_width;
  int         train_width;

  buttons_bar = gtk_toolbar_new();

  //Create the save train icon
  system_colormap = gdk_colormap_get_system();
  save_icon = gtk_widget_render_icon(buttons_bar,
				     GTK_STOCK_SAVE,
				     GTK_ICON_SIZE_MENU,
				     NULL);
  train_save_icon = gdk_pixmap_new(NULL,
				   gdk_pixbuf_get_width(save_icon),
				   gdk_pixbuf_get_height(save_icon),
				   system_colormap->visual->depth); // This is very ugly and should be improved.
  gdk_drawable_set_colormap(train_save_icon,
			    system_colormap);
  icon_gc = gdk_gc_new(train_save_icon);
  gdk_pixbuf_render_to_drawable(save_icon,  // Change to gdk_draw_pixbuf when moving to 2.2
				train_save_icon,
				icon_gc,
				0,0,
				0,0,
				-1, -1,
				GDK_RGB_DITHER_NONE,
				0, 0);
  g_object_unref(G_OBJECT(save_icon));

  gdk_drawable_get_size(train_save_icon,
			&icon_width,
			NULL);
  gdk_drawable_get_size(e_train_pmap[0],
			&train_width,
			NULL);

  gdk_draw_drawable(train_save_icon,
		    icon_gc,
		    e_train_pmap[0],
		    0, 1,
		    icon_width - train_width, 0,
		    -1, 7);
  train_save_image = gtk_image_new_from_pixmap(train_save_icon,
					       NULL);
  //End create icon


  gtk_toolbar_append_item(GTK_TOOLBAR(buttons_bar),
			  NULL,
			  "Save full schedule in html file",
			  NULL,
			  gtk_image_new_from_stock(GTK_STOCK_SAVE,
						   GTK_ICON_SIZE_MENU),
			  G_CALLBACK(save_schedule_cmd),
			  NULL);

  gtk_toolbar_append_item(GTK_TOOLBAR(buttons_bar),
			  NULL,
			  "Save selected train schedule in html file",
			  NULL,
			  train_save_image,
			  G_CALLBACK(save_train_cmd),
			  NULL);

  gtk_toolbar_append_space(GTK_TOOLBAR(buttons_bar));

  toggle_button = gtk_toggle_button_new_with_label("Full");
  gtk_button_set_relief(GTK_BUTTON(toggle_button),
			GTK_RELIEF_NONE);
  gtk_container_set_border_width(GTK_CONTAINER(toggle_button),
				 3);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle_button),
			       !FULL_SCHEDULE);
  g_signal_connect(toggle_button,
		   "toggled",
		   G_CALLBACK(full_button_toggle),
		   NULL);
  gtk_toolbar_append_element(GTK_TOOLBAR(buttons_bar),
			     GTK_TOOLBAR_CHILD_WIDGET, //GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
			     toggle_button,
			     NULL,
			     "Displays/hides the full schedule view",
			     NULL,
			     NULL,
			     NULL,
			     NULL);

  toggle_button = gtk_toggle_button_new_with_label("Active");
  gtk_button_set_relief(GTK_BUTTON(toggle_button),
			GTK_RELIEF_NONE);
  gtk_container_set_border_width(GTK_CONTAINER(toggle_button),
				 3);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle_button),
			       !ACTIVE_SCHEDULE);
  g_signal_connect(toggle_button,
		   "toggled",
		   G_CALLBACK(active_button_toggle),
		   NULL);
  gtk_toolbar_append_element(GTK_TOOLBAR(buttons_bar),
			     GTK_TOOLBAR_CHILD_WIDGET, //GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
			     toggle_button,
			     NULL,
			     "Displays/hides the active trains schedule view",
			     NULL,
			     NULL,
			     NULL,
			     NULL);

  toggle_button = gtk_toggle_button_new_with_label("Waiting");
  gtk_button_set_relief(GTK_BUTTON(toggle_button),
			GTK_RELIEF_NONE);
  gtk_container_set_border_width(GTK_CONTAINER(toggle_button),
				 3);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle_button),
			       !WAITING_SCHEDULE);
  g_signal_connect(toggle_button,
		   "toggled",
		   G_CALLBACK(waiting_button_toggle),
		   NULL);
  gtk_toolbar_append_element(GTK_TOOLBAR(buttons_bar),
			     GTK_TOOLBAR_CHILD_WIDGET, //GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
			     toggle_button,
			     NULL,
			     "Displays/hides the waiting trains schedule view",
			     NULL,
			     NULL,
			     NULL,
			     NULL);

  toggle_button = gtk_toggle_button_new_with_label("Train");
  gtk_button_set_relief(GTK_BUTTON(toggle_button),
			GTK_RELIEF_NONE);
  gtk_container_set_border_width(GTK_CONTAINER(toggle_button),
				 3);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle_button),
			       default_show_selected_train);
  g_signal_connect(toggle_button,
		   "toggled",
		   G_CALLBACK(selected_train_button_toggle),
		   NULL);
  gtk_toolbar_append_element(GTK_TOOLBAR(buttons_bar),
			     GTK_TOOLBAR_CHILD_WIDGET, //GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
			     toggle_button,
			     NULL,
			     "Displays/hides the selected train schedule view",
			     NULL,
			     NULL,
			     NULL,
			     NULL);

  gtk_box_pack_start(GTK_BOX(parent),
		     buttons_bar,
		     FALSE,
		     TRUE,
		     0);
  gtk_widget_show_all(buttons_bar);
}

void  schedule_row_selected(GtkTreeView *tree_view,
			    gpointer active_schedule_data)
{
  GtkTreePath *selected_row;
  GtkTreeViewColumn *focus_column;
  gchar *row_nr;
  gint view, j;

  for (view = 0; view < NR_SCHEDULES && active_schedule_data != schedule_data[view]; ++view);

  gtk_tree_view_get_cursor(tree_view,
			   &selected_row,
			   &focus_column);
  row_nr = gtk_tree_path_to_string(selected_row);
  g_free(row_nr);

  if (selected_row != NULL && (view == FULL_SCHEDULE    ||
    			       view == WAITING_SCHEDULE ||
    			       view == ACTIVE_SCHEDULE)){
    GtkTreeIter tmp_iter;
    Train *t;
    GValue train_name;

    // I am not really satisfied with the way the GValue is reset.
    G_VALUE_TYPE(&train_name) = G_TYPE_INVALID;
    gtk_tree_model_get_iter(GTK_TREE_MODEL(schedule_data[view]),
			    &tmp_iter,
			    selected_row);
    gtk_tree_model_get_value(GTK_TREE_MODEL(schedule_data[view]),
			     &tmp_iter,
			     TRAIN_ID,
			     &train_name);
    t = findTrainNamed(g_value_get_string(&train_name));
    selected_train = t;
    g_value_unset(&train_name);

    if (t) {
      TrainStop *train_stop;
      gchar min_stop_buff[256];
      gchar arrival_time_buff[256];
      gchar buff[256];
      gchar *p;

      sprintf(buff, "%d", t->type + 1);
      gtk_label_set_text(GTK_LABEL(train_info_element[0]),
			 t->name ? t->name: "");
      gtk_label_set_text(GTK_LABEL(train_info_element[1]),
			 buff);
      gtk_label_set_text(GTK_LABEL(train_info_element[2]),
			 t->waitfor ? t->waitfor: "");
      gtk_label_set_text(GTK_LABEL(train_info_element[3]),
			 t->stock ? t->stock: "");
      
      gtk_list_store_clear(train_schedule_data);
      
      gtk_list_store_append(train_schedule_data,
			    &tmp_iter);
      gtk_list_store_set(train_schedule_data,
			 &tmp_iter,
			 0, t->entrance ? t->entrance : "",
			 1, format_time(t->timein),
			 -1);
      for(train_stop = t->stops; train_stop; train_stop = train_stop->next){
	strcpy(buff, train_stop->station);
	if((p = strchr(buff, '@')))
	  *p = 0;
	if(!findStationNamed(buff))
	  continue;
	
	sprintf(min_stop_buff, "%d", train_stop->minstop);
	strcpy(arrival_time_buff, format_time(train_stop->arrival));
	gtk_list_store_append(train_schedule_data,
			      &tmp_iter);
	gtk_list_store_set(train_schedule_data,
			   &tmp_iter,
			   0, train_stop->station,
			   1, arrival_time_buff,
			   2, format_time(train_stop->departure),
			   3, min_stop_buff,
			   -1);
      }
      gtk_list_store_append(train_schedule_data,
			    &tmp_iter);
      gtk_list_store_set(train_schedule_data,
			 &tmp_iter,
			 0, t->exit ? t->exit : "",
			 1, format_time(t->timeout),
			 -1);
    }
  }

  if (view == ASSIGN_SCHEDULE)
    assign_selected = selected_row;
}

void  create_schedule_area(GtkWidget **widget,
			   TrainListViews view,
			   gboolean show)
{
  GtkWidget *schedule_widget;
  gint i;

  schedule_list[view] = gtk_scrolled_window_new(NULL,
						NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(schedule_list[view]),
				 GTK_POLICY_AUTOMATIC,
				 GTK_POLICY_ALWAYS);
  
  schedule_widget = gtk_tree_view_new_with_model(GTK_TREE_MODEL(schedule_data[view]));
  
  for (i = 0; i < NR_SCHEDULE_TEXT_COLUMNS; ++i)
    gtk_tree_view_append_column(GTK_TREE_VIEW(schedule_widget),
				gtk_tree_view_column_new_with_attributes(schedule_colum_titles[i],
									 schedule_text_renderer,
									 "text", i, 
									 NULL));

  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(schedule_widget),
				    TRUE);

  gtk_tree_view_columns_autosize(GTK_TREE_VIEW(schedule_widget));
  gtk_widget_show(schedule_widget);
  gtk_container_add(GTK_CONTAINER(schedule_list[view]),
		    schedule_widget);
  *widget = schedule_list[view];

  g_signal_connect(schedule_widget,
		   "cursor-changed",
		   G_CALLBACK(schedule_row_selected),
		   (gpointer) schedule_data[view]);

  if (show)
    gtk_widget_show(schedule_list[view]);
}

static gchar *train_label_default[4][2] = {{"Train name:", ""},
					   {"Train type:", ""},
					   {"Wait for train:", ""},
					   {"Stock for train:",""}};
static gchar *train_table_headings[4] = {"Station",
					 "Arrival",
					 "Departure",
					 "Min.Stop (sec)"};

void  create_selected_train_area(GtkWidget **selected_widget)
{
  GtkWidget *train_info_container;
  GtkWidget *scrolled_window;
  GtkWidget *vbox;
  gint i, j;

  selected_train_list = gtk_hbox_new(FALSE,
				     1);
  *selected_widget = selected_train_list;

  vbox = gtk_vbox_new(FALSE,
		      1);
  gtk_box_pack_start(GTK_BOX(selected_train_list),
		     vbox,
		     FALSE,
		     FALSE,
		     0);

  train_info_container = gtk_table_new(2,
				       4, 
				       FALSE);
  gtk_table_set_row_spacings(GTK_TABLE(train_info_container),
			     3);
  gtk_table_set_col_spacings(GTK_TABLE(train_info_container),
			     3);
  gtk_box_pack_start(GTK_BOX(vbox),
		     train_info_container,
		     FALSE,
		     FALSE,
		     0);
  gtk_box_pack_start(GTK_BOX(vbox),
		     gtk_label_new(""),
		     FALSE,
		     FALSE,
		     0);
  for (j = 0; j < 4; ++j) {
    GtkWidget *status_element;
    GtkWidget *frame_element;

    status_element = gtk_label_new(train_label_default[j][0]);
    gtk_misc_set_alignment(GTK_MISC(status_element),
			   0.0,
			   0.5);
    gtk_table_attach_defaults(GTK_TABLE(train_info_container),
			      status_element,
			      0,
			      1,
			      j,
			      j + 1);

    frame_element = gtk_frame_new(NULL);
    gtk_frame_set_shadow_type(GTK_FRAME(frame_element),
			      GTK_SHADOW_ETCHED_IN);
    gtk_table_attach_defaults(GTK_TABLE(train_info_container),
			      frame_element,
			      1,
			      2,
			      j,
			      j + 1);
    status_element = gtk_label_new(train_label_default[j][1]);
    gtk_misc_set_alignment(GTK_MISC(status_element),
			   0.0,
			   0.5);
    g_object_set(status_element,
		 "width-request", 100,
		 NULL);
    gtk_container_add(GTK_CONTAINER(frame_element),
		      status_element);
    train_info_element[j] = status_element;
  }

  scrolled_window = gtk_scrolled_window_new(NULL,
					    NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
				 GTK_POLICY_AUTOMATIC,
				 GTK_POLICY_ALWAYS);
  gtk_box_pack_start(GTK_BOX(selected_train_list),
		     scrolled_window,
		     TRUE,
		     TRUE,
		     0);
  {
    GtkWidget *schedule_widget;
    gint i;
    
    //    if (train_schedule_data)
    //      gtk_list_store_clear(train_schedule_data);
    //    else
    train_schedule_data = gtk_list_store_new(4,
					     G_TYPE_STRING,
					     G_TYPE_STRING,
					     G_TYPE_STRING,
					     G_TYPE_STRING);

    schedule_widget = gtk_tree_view_new_with_model(GTK_TREE_MODEL(train_schedule_data));
  
    for (i = 0; i < 4; ++i)
      gtk_tree_view_append_column(GTK_TREE_VIEW(schedule_widget),
				  gtk_tree_view_column_new_with_attributes(train_table_headings[i],
									   schedule_text_renderer,
									   "text", i, 
									   NULL));

    gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(schedule_widget),
				      TRUE);

    gtk_tree_view_columns_autosize(GTK_TREE_VIEW(schedule_widget));
    gtk_container_add(GTK_CONTAINER(scrolled_window),
		      schedule_widget);

  }

  if (default_show_selected_train)
    gtk_widget_show_all(selected_train_list);
}

void  close_schedule(GtkWidget* source,
		     gpointer userdata)
{
  gint i;

  gtk_widget_destroy(schedule_window);
  schedule_window = NULL;
}

int	create_schedule(int assign)
{                                  
  GtkWidget *vbox;
  gint i;

  //if(assign)
  //    memset(&asgn_struct, 0, sizeof(asgn_struct));
  if(schedule_window)			/* no reentrancy. Schedule window need to be set to NULL at destruction*/
    return 0;

  // Set text renderer if unset.
  if (!schedule_text_renderer ||
      !GTK_IS_CELL_RENDERER(schedule_text_renderer))
    schedule_text_renderer = gtk_cell_renderer_text_new();

  schedule_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_size_request(GTK_WIDGET(schedule_window),
			      832,
			      600);

  //  if(assign)
  //    gtk_window_set_modal(GTK_WINDOW(window),TRUE);

  gtk_window_set_title(GTK_WINDOW(schedule_window),
		       schedule_window_view_title); //assign ? "Departing trains" : "Train Schedule");
  g_signal_connect(GTK_OBJECT(schedule_window),
		   "destroy",
		   G_CALLBACK(close_schedule),
		   NULL);

  vbox = gtk_vbox_new(FALSE,
		      1);
  gtk_container_add(GTK_CONTAINER(schedule_window),
		    vbox);
  gtk_widget_show(vbox);

  create_schedule_button_bar(vbox);

  for (i = 0; i < NR_TRAIN_LIST_VIEWS; ++i)
    create_schedule_list(i,
			 NULL);

  // Create the schedule areas in paned containers
  {
    GtkWidget *paned_top;
    GtkWidget *paned;
    GtkWidget *tmp1;
    GtkWidget *tmp2;

    paned_top = gtk_vpaned_new ();
    tmp1 = gtk_vpaned_new ();
    create_selected_train_area(&tmp2);
    gtk_paned_pack1 (GTK_PANED (paned_top),
		     tmp1, 
		     TRUE,
		     FALSE);
    gtk_paned_pack2 (GTK_PANED (paned_top), 
		     tmp2,
		     TRUE,
		     FALSE);
    paned = tmp1;
    gtk_widget_show(paned);

    create_schedule_area(&tmp1,        
			 FULL_SCHEDULE,
			 TRUE);
    tmp2 = gtk_vpaned_new ();
    gtk_paned_pack1 (GTK_PANED (paned),
		     tmp1, 
		     TRUE,
		     FALSE);
    gtk_paned_pack2 (GTK_PANED (paned), 
		     tmp2,
		     TRUE,
		     FALSE);
    paned = tmp2;
    gtk_widget_show(paned);

    create_schedule_area(&tmp1,        
			 ACTIVE_SCHEDULE,
			 FALSE);
    create_schedule_area(&tmp2,        
			 WAITING_SCHEDULE,
			 FALSE);
    gtk_paned_pack1 (GTK_PANED (paned),
		     tmp1, 
		     TRUE,
		     FALSE);
    gtk_paned_pack2 (GTK_PANED (paned), 
		     tmp2,
		     TRUE,
		     FALSE);
    gtk_widget_show(paned_top);

    gtk_box_pack_start((GtkBox*) vbox,
		       paned_top,
		       TRUE,
		       TRUE,
		       0);
  }

  gtk_widget_show(schedule_window);
  return(0);
}

void  show_schedule_cmd(void)
{
  trainsim_cmd("sched");
}

void  do_assign_train(GtkWidget * source,
		      gpointer *old_train)
{
  GtkTreeIter tmp_iter;
  Train *t;
  GValue train_name;

  if (!assign_selected)
    return;
  
  // I am not really satisfied with the way the GValue is reset.
  G_VALUE_TYPE(&train_name) = G_TYPE_INVALID;
  gtk_tree_model_get_iter(GTK_TREE_MODEL(schedule_data[ASSIGN_SCHEDULE]),
			  &tmp_iter,
			  assign_selected);
  gtk_tree_model_get_value(GTK_TREE_MODEL(schedule_data[ASSIGN_SCHEDULE]),
			   &tmp_iter,
			   TRAIN_ID,
			   &train_name);
  t = findTrainNamed(g_value_get_string(&train_name));
  g_value_unset(&train_name);
  assign_train(t,
	       (Train*) old_train);
  gtk_main_quit();
}

void  close_assign_window(GtkWidget *window)
{
  assign_selected = NULL;
  gtk_widget_destroy(window);
}

int   gtk_assign_dialog(Train *t)
{
  GtkWidget *assign_schedule_widget;
  GtkWidget *assign_window;
  GtkWidget *vbox, *hbox;

  // Set text renderer if unset.
  if (!schedule_text_renderer ||
      !GTK_IS_CELL_RENDERER(schedule_text_renderer))
    schedule_text_renderer = gtk_cell_renderer_text_new();

  create_schedule_list(ASSIGN_SCHEDULE,
		       t);

  assign_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_modal(GTK_WINDOW(assign_window),TRUE);
  gtk_widget_set_size_request(GTK_WIDGET(assign_window),
			      832,
			      400);
  gtk_window_set_title(GTK_WINDOW(assign_window),
		       "Departing trains");  // use constant / define.
  g_signal_connect(GTK_OBJECT(assign_window),
		   "destroy",
		   G_CALLBACK(close_assign_window),
		   NULL);

  vbox = gtk_vbox_new(FALSE,
		      1);
  gtk_container_add(GTK_CONTAINER (assign_window),
		    vbox);

  hbox = gtk_hbox_new(FALSE, 
		      1);
  gtk_box_pack_start(GTK_BOX(vbox),
		     hbox,
		     FALSE,
		     FALSE,
		     0);
  

  {
    GtkWidget *buttons_bar;
    
    buttons_bar = gtk_toolbar_new();
    gtk_toolbar_append_item(GTK_TOOLBAR(buttons_bar),
			    NULL,
			    "Assigns selected train",
			    NULL,
			    gtk_image_new_from_stock(GTK_STOCK_APPLY,
						     GTK_ICON_SIZE_MENU),
			    G_CALLBACK(do_assign_train),
			    (gpointer) t);

    gtk_toolbar_append_space(GTK_TOOLBAR(buttons_bar));

    gtk_box_pack_start(GTK_BOX(hbox),
		       buttons_bar,
		       FALSE,
		       TRUE,
		       0);
  }
  {
    GtkWidget *label_widget;
    GtkWidget *frame_widget;

    label_widget = gtk_label_new(train_label_default[0][0]);
    gtk_misc_set_padding(GTK_MISC(label_widget),
			 3,
			 0);
    gtk_box_pack_start(GTK_BOX(hbox),
		       label_widget,
		       FALSE,
		       FALSE,
		       0);
   
    frame_widget = gtk_frame_new(NULL);
    gtk_frame_set_shadow_type(GTK_FRAME(frame_widget),
			      GTK_SHADOW_ETCHED_IN);
    label_widget = gtk_label_new(t->name ? t->name: "");
    gtk_misc_set_alignment(GTK_MISC(label_widget),
			   0.0,
			   0.5);
    g_object_set(label_widget,
		 "width-request", 100,
		 NULL);
    gtk_container_add(GTK_CONTAINER(frame_widget),
		      label_widget);
    gtk_box_pack_start(GTK_BOX(hbox),
		       frame_widget,
		       FALSE,
		       FALSE,
		       0);
 
    label_widget = gtk_label_new(train_label_default[3][0]);
    gtk_misc_set_padding(GTK_MISC(label_widget),
			 3,
			 0);
    gtk_box_pack_start(GTK_BOX(hbox),
		       label_widget,
		       FALSE,
		       FALSE,
		       0);
   
    frame_widget = gtk_frame_new(NULL);
    gtk_frame_set_shadow_type(GTK_FRAME(frame_widget),
			      GTK_SHADOW_ETCHED_IN);
    label_widget = gtk_label_new(t->stock ? t->stock: "");
    gtk_misc_set_alignment(GTK_MISC(label_widget),
			   0.0,
			   0.5);
    g_object_set(label_widget,
		 "width-request", 100,
		 NULL);
    gtk_container_add(GTK_CONTAINER(frame_widget),
		      label_widget);
    gtk_box_pack_start(GTK_BOX(hbox),
		       frame_widget,
		       FALSE,
		       FALSE,
		       0);

    gtk_box_pack_start(GTK_BOX(hbox),
		       gtk_label_new(""),
		       TRUE,
		       TRUE,
		       0);
  }
  create_schedule_area(&assign_schedule_widget,
		       ASSIGN_SCHEDULE,
		       TRUE);
  gtk_box_pack_start(GTK_BOX(vbox),
		     assign_schedule_widget,
		     TRUE,
		     TRUE,
		     0);
  gtk_widget_show_all(assign_window);

  gtk_main();

  gtk_widget_destroy(assign_window);

  if (assign_selected == NULL) // Might always be true depending on when the destroy sgnal is sent.
    return 0;
  
  assign_selected = NULL;
  return 1;

}
