/*	gtk_select_day_dialog.c - Created by Giampiero Caprino
Modified by Erik Kuiper

This file is part of Train Director

Train Director is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

Train Director is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Train Director; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include <stdio.h> //For trsim.h
#include "trsim.h"

struct	DayData {
  GtkWidget *window;
  GtkWidget *buttons[8];
  int	which;
};
static	struct DayData day_data;
static	char	*days[] = { "Monday", 
			    "Tuesday",
			    "Wednesday", 
			    "Thursday",
			    "Friday",
			    "Saturday",
			    "Sunday",
			    0 };

static	void	day_selected(gpointer button, 
			     gpointer data)
{
  int	i;

  for(i = 0; i < 7; ++i)
    if(data == day_data.buttons[i])
      break;
  if(i == 7)
    day_data.which = 0;
  else
    day_data.which = i + 1;
}

static	void	day_continue(gpointer button,
			     gpointer data)
{
  if(!day_data.window)
    return;
  gtk_widget_destroy(day_data.window);
  day_data.window = 0;
  if(day_data.which)
    run_day = 1 << (day_data.which - 1);
  else
    run_day = 0;
  gtk_main_quit();
}

void	gtk_select_day_dialog(void)
{
  GtkWidget *window = NULL;
  GtkWidget *box1, *box2;
  GtkWidget *separator;
  GtkWidget *button;
  char	buff[256];
  int	i;

  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(window), "Day Selection");
  gtk_window_set_modal(GTK_WINDOW(window),TRUE);
  box1 = gtk_vbox_new(FALSE,5);
  gtk_container_add(GTK_CONTAINER(window), box1);

  box2 = gtk_vbox_new (FALSE, 10);
  gtk_container_set_border_width (GTK_CONTAINER (box2), 10);
  gtk_box_pack_start (GTK_BOX (box1), box2, TRUE, TRUE, 0);

  separator = gtk_label_new("Not all trains run every day of the week.\nWhich day do you want to simulate?");
  gtk_box_pack_start (GTK_BOX (box2), separator, TRUE, TRUE, 0);

  button = gtk_radio_button_new_with_label (NULL, days[0]);
  gtk_box_pack_start (GTK_BOX (box2), button, TRUE, TRUE, 0);
  day_data.buttons[0] = button;
  day_data.which = 1;		    /* default is first day */
  for(i = 1; days[i]; ++i) {
    button = gtk_radio_button_new_with_label (gtk_radio_button_get_group (GTK_RADIO_BUTTON (button)),
					      days[i]);
    g_signal_connect (GTK_OBJECT (button),
		      "clicked",
		      G_CALLBACK(day_selected),
		      GTK_OBJECT (button));
    gtk_box_pack_start (GTK_BOX (box2), button, TRUE, TRUE, 0);
    day_data.buttons[i] = button;
  }

  separator = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 0);

  box2 = gtk_vbox_new (FALSE, 10);
  gtk_container_set_border_width (GTK_CONTAINER (box2), 10);
  gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, TRUE, 0);

  button = gtk_button_new_with_label ("Ok");
  g_signal_connect (GTK_OBJECT (button),
		    "clicked",
		    G_CALLBACK(day_continue),
		    GTK_OBJECT (window));
  gtk_box_pack_start (GTK_BOX (box2), button, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (button);
  day_data.window = window;
  gtk_widget_show_all(window);
  gtk_main();
}
